using View_by_Distance.Shared.Models.Stateless;

namespace View_by_Distance.Property.Models.Stateless;

internal class Result
{

    internal static string GetRelativePath(Configuration configuration, string path)
    {
        string result = Shared.Models.Stateless.Methods.IPath.GetRelativePath(path, configuration.RootDirectory.Length);
        return result;
    }

    internal static string GetResultsGroupDirectory(Configuration configuration, string description, bool create)
    {
        string result = Path.Combine($"{configuration.RootDirectory} - Results", description.Replace("_", ") "));
        if (create && !Directory.Exists(result))
            _ = Directory.CreateDirectory(result);
        return result;
    }

    internal static string GetResultsDateGroupDirectory(Configuration configuration, string description)
    {
        string result = Path.Combine(GetResultsGroupDirectory(configuration, description, create: true), configuration.DateGroup);
        if (!Directory.Exists(result))
            _ = Directory.CreateDirectory(result);
        return result;
    }

    internal static string GetResultsDateGroupDirectory(Configuration configuration, string description, string jsonGroup)
    {
        string result = Path.Combine(GetResultsDateGroupDirectory(configuration, description), jsonGroup);
        if (!Directory.Exists(result))
            _ = Directory.CreateDirectory(result);
        return result;
    }

    internal static string GetResultsFullGroupDirectory(Configuration configuration, Model? model, PredictorModel? predictorModel, string description, string outputResolution, bool includeResizeGroup, bool includeModel, bool includePredictorModel)
    {
        string result = GetResultsDateGroupDirectory(configuration, description);
        if (includeResizeGroup)
            result = Path.Combine(result, outputResolution);
        if (includeModel && includePredictorModel)
        {
            string modelName;
            string predictorModelName;
            if (model is null)
                modelName = Model.Hog.ToString();
            else
                modelName = model.Value.ToString();
            if (predictorModel is null)
                predictorModelName = PredictorModel.Large.ToString();
            else
                predictorModelName = predictorModel.Value.ToString();
            string dateGroupDirectory = string.Concat(outputResolution.Replace(" ", string.Empty), " - ", modelName, " - ", predictorModelName);
            result = Path.Combine(result, dateGroupDirectory);
        }
        else if (includeModel)
            throw new Exception();
        else if (includePredictorModel)
            throw new Exception();
        if (!Directory.Exists(result))
            _ = Directory.CreateDirectory(result);
        return result;
    }

    private static void CheckContent(Configuration configuration, string dateGroupDirectory, string contentDescription, string result)
    {
        string checkDirectory;
        checkDirectory = Path.Combine(dateGroupDirectory, configuration.ResultContent, configuration.ResultAllInOne);
        if (!Directory.Exists(checkDirectory))
            _ = Directory.CreateDirectory(checkDirectory);
        string contentDirectory = new(result.Replace("<>", configuration.ResultContent));
        if (!Directory.Exists(contentDirectory))
            _ = Directory.CreateDirectory(contentDirectory);
        checkDirectory = Path.Combine(dateGroupDirectory, string.Concat("() - ", contentDescription));
        if (!Directory.Exists(checkDirectory))
            _ = Directory.CreateDirectory(checkDirectory);
    }

    private static void CheckSingleton(Configuration configuration, string dateGroupDirectory, string singletonDescription, bool converted, string result)
    {
        string checkDirectory;
        checkDirectory = Path.Combine(dateGroupDirectory, configuration.ResultSingleton, configuration.ResultAllInOne);
        if (!Directory.Exists(checkDirectory))
            _ = Directory.CreateDirectory(checkDirectory);
        if (!converted)
        {
            string singletonDirectory = new(result.Replace("<>", configuration.ResultSingleton));
            if (!Directory.Exists(singletonDirectory))
                _ = Directory.CreateDirectory(singletonDirectory);
        }
        checkDirectory = Path.Combine(dateGroupDirectory, string.Concat("{} - ", singletonDescription));
        if (!Directory.Exists(checkDirectory))
            _ = Directory.CreateDirectory(checkDirectory);
    }

    private static void CheckCollection(Configuration configuration, string dateGroupDirectory, string collectionDescription, bool converted, string result)
    {
        string checkDirectory = Path.Combine(dateGroupDirectory, configuration.ResultCollection, configuration.ResultAllInOne);
        if (!Directory.Exists(checkDirectory))
            _ = Directory.CreateDirectory(checkDirectory);
        if (!converted)
        {
            string collectionDirectory = new(result.Replace("<>", configuration.ResultCollection));
            if (!Directory.Exists(collectionDirectory))
                _ = Directory.CreateDirectory(collectionDirectory);
        }
        checkDirectory = Path.Combine(dateGroupDirectory, string.Concat("[] - ", collectionDescription));
        if (!Directory.Exists(checkDirectory))
            _ = Directory.CreateDirectory(checkDirectory);
    }

    internal static List<string> GetDirectoryInfoCollection(Configuration configuration, string sourceDirectory, string dateGroupDirectory, string contentDescription, string singletonDescription, string collectionDescription, bool converted)
    {
        List<string> results = new();
        string sourceDirectorySegment = GetRelativePath(configuration, sourceDirectory);
        string result = string.Concat(Path.Combine(dateGroupDirectory, "<>"), sourceDirectorySegment);
        if (!string.IsNullOrEmpty(contentDescription))
            CheckContent(configuration, dateGroupDirectory, contentDescription, result);
        if (!string.IsNullOrEmpty(singletonDescription))
            CheckSingleton(configuration, dateGroupDirectory, singletonDescription, converted, result);
        if (!string.IsNullOrEmpty(collectionDescription))
            CheckCollection(configuration, dateGroupDirectory, collectionDescription, converted, result);
        results.Add(result);
        return results;
    }

    internal static List<string> GetDirectoryInfoCollection(Configuration configuration, Model? model, PredictorModel? predictorModel, string sourceDirectory, string description, string outputResolution, bool includeResizeGroup, bool includeModel, bool includePredictorModel, string contentDescription, string singletonDescription, string collectionDescription)
    {
        List<string> results;
        bool converted = false;
        string dateGroupDirectory = GetResultsFullGroupDirectory(configuration, model, predictorModel, description, outputResolution, includeResizeGroup, includeModel, includePredictorModel);
        results = GetDirectoryInfoCollection(configuration, sourceDirectory, dateGroupDirectory, contentDescription, singletonDescription, collectionDescription, converted);
        return results;
    }

}