using System.Text.Json;
using System.Text.Json.Serialization;

namespace View_by_Distance.Shared.Models;

public class Location : Properties.ILocation, IEquatable<Location>
{

    protected int _Bottom;
    protected double _Confidence;
    protected int _Left;
    protected readonly int? _NormalizedPixelPercentage;
    protected int _Right;
    protected int _Top;
    public double Confidence => _Confidence;
    public int Bottom => _Bottom;
    public int Left => _Left;
    public int? NormalizedPixelPercentage => _NormalizedPixelPercentage;
    public int Right => _Right;
    public int Top => _Top;

    [JsonConstructor]
    public Location(int bottom, double confidence, int left, int? normalizedPixelPercentage, int right, int top)
    {
        _Confidence = confidence;
        _Bottom = bottom;
        _Left = left;
        _NormalizedPixelPercentage = normalizedPixelPercentage;
        _Right = right;
        _Top = top;
        Stateless.Methods.Location.Check(bottom, left, normalizedPixelPercentage, right, top, zCount: 1);
    }

    public Location(double confidence, int height, Location location, int locationDigits, int locationFactor, int width, int zCount) :
        this(location.Bottom, confidence, location.Left, Stateless.Methods.Location.GetNormalizedPixelPercentage(location.Bottom, height, location.Left, locationDigits, locationFactor, location.Right, location.Top, width, zCount), location.Right, location.Top) =>
        Stateless.Methods.Location.Check(_Bottom, _Left, _NormalizedPixelPercentage, _Right, _Top, zCount);

    public Location(int bottom, double confidence, int height, int left, int locationDigits, int locationFactor, int right, int top, int width, int zCount) :
        this(bottom, confidence, left, Stateless.Methods.Location.GetNormalizedPixelPercentage(bottom, height, left, locationDigits, locationFactor, right, top, width, zCount), right, top) =>
        Stateless.Methods.Location.Check(_Bottom, height, _Left, _NormalizedPixelPercentage, _Right, _Top, width, zCount);

    public Location(double confidence, int factor, int height, Location location, int locationDigits, int locationFactor, int width, int zCount)
    {
        int x = (location.Right - location.Left) / factor;
        int y = (location.Bottom - location.Top) / factor;
        int bottom = Math.Min(location.Bottom + y, height);
        int left = Math.Max(location.Left - x, 0);
        int right = Math.Min(location.Right + x, width);
        int top = Math.Max(location.Top - y, 0);
        int normalizedPixelPercentage = Stateless.Methods.Location.GetNormalizedPixelPercentage(location.Bottom, height, location.Left, locationDigits, locationFactor, location.Right, location.Top, width, zCount);
        Stateless.Methods.Location.Check(bottom, left, _NormalizedPixelPercentage, right, top, zCount);
        _Confidence = confidence;
        _Bottom = bottom;
        _Left = left;
        _NormalizedPixelPercentage = normalizedPixelPercentage;
        _Right = right;
        _Top = top;
    }

    public override bool Equals(object? obj) => Equals(obj as Location);

    public override string ToString()
    {
        string result = JsonSerializer.Serialize(this, new JsonSerializerOptions() { WriteIndented = true });
        return result;
    }

    public override int GetHashCode()
    {
        int hashCode = -773114317;
        hashCode = hashCode * -1521134295 + _Bottom.GetHashCode();
        hashCode = hashCode * -1521134295 + _Left.GetHashCode();
        hashCode = hashCode * -1521134295 + _Right.GetHashCode();
        hashCode = hashCode * -1521134295 + _Top.GetHashCode();
        return hashCode;
    }

    public bool Equals(Location? location)
    {
        return location is not null
            && _Bottom == location.Bottom
            && _Left == location.Left
            && _Right == location.Right
            && _Top == location.Top;
    }

    public static bool operator ==(Location location1, Location location2) => EqualityComparer<Location>.Default.Equals(location1, location2);

    public static bool operator !=(Location location1, Location location2) => !(location1 == location2);

}